import scala.io.Source
import java.io.File
import java.io.PrintWriter
import scala.collection.mutable.ArrayBuffer

class DiskRango(val entregadores:Circular[Entregador], val items:List[Item], var clientes:ArrayBuffer[Cliente], var pedidos:ArrayBuffer[Pedido]) {
  def this(entregadores:Circular[Entregador], items:List[Item]) = {
    this(entregadores, items, new ArrayBuffer[Cliente], new ArrayBuffer[Pedido])
  }

  def hasCliente(telefone:String) = {
    val index = clientes.indexWhere(_.telefone == telefone)
    if(index == -1) false
    else true
  }
  def indexCliente(telefone:String) = {
    clientes.indexWhere(_.telefone == telefone)
  }
  def loadCliente(index:Int) = {
    clientes(index)
  }
  @scala.annotation.tailrec
  final def buscaCliente(telefone:String):Cliente = {
    if(!hasCliente(telefone)) {
      println("Cliente não cadastrado. Novo cadastro:")
      val cliente = Cliente.cadastro(telefone)
      clientes.append(cliente)
    }
    val cliente = loadCliente(indexCliente(telefone))
    println("Cliente cadastrado: " + cliente.nome)
    println("[C]onfirma ou [R]ecadastra?")
    val resposta = readLine
    while(resposta != "R" && resposta != "r" && resposta != "C" && resposta != "c"){
      println("Cliente cadastrado: " + cliente.nome + " [C]onfirma ou [R]ecadastra?")
      val resposta = readLine
    }
    if(resposta == "R" | resposta == "r"){
      println("Recadastrando cliente (telefone: " + cliente.telefone + ")")
      Cliente.cadastro(cliente)
      buscaCliente(cliente.telefone)
    }else cliente
  }

  def hasItem(cod:String) = {
    val index = items.indexWhere(_.cod == cod)
    if(index == -1) false
    else true
  }
  def indexItem(cod:String) = {
    items.indexWhere(_.cod == cod)
  }
  def loadItem(cod:String) = {
    items(indexItem(cod))
  }
}


object DiskRango extends App {
  val usage = """
    Usage: DiskRango [options]
      -e <file> | --entregadores <file>
        lista de entregadores em formato csv
      -c <file> | --cardapio <file>
        cardapio em formato csv
      -p <file> | --pedidos <file>
        pedidos em formato csv
      -h | --help

      Not Implemented
       -cl <file> | --clientes <file>
        clientes em formato csv
  """
  if (args.length == 0){
    println(usage)
    sys.exit(0)
  }
  val arglist = args.toList
  type OptionMap = Map[Symbol, Any]

  def nextOption(map : OptionMap, list: List[String]) : OptionMap = {
    //def isSwitch(s : String) = (s(0) == '-')
    list match {
      case Nil => map
      case "-h" :: tail => println(usage)
                   sys.exit(0)
      case "-e" :: value :: tail => nextOption(map ++ Map('entregadores -> value), tail)
      case "--entregadores" :: value :: tail => nextOption(map ++ Map('entregadores -> value), tail)
      case "-c" :: value :: tail => nextOption(map ++ Map('cardapio -> value), tail)
      case "--cardapio" :: value :: tail => nextOption(map ++ Map('cardapio -> value), tail)
      case "-cl" :: value :: tail => nextOption(map ++ Map('clientes -> value), tail)
      case "--clientes" :: value :: tail => nextOption(map ++ Map('clientes -> value), tail)
      case "-p" :: value :: tail => nextOption(map ++ Map('pedidos -> value), tail)
      case "--pedidos" :: value :: tail => nextOption(map ++ Map('pedidos -> value), tail)
      //case string :: tail =>  nextOption(map ++ Map('infile -> string), tail)
      //case string :: opt2 :: tail if isSwitch(opt2) => nextOption(map ++ Map('infile -> string), list.tail)
      //case string :: Nil =>  nextOption(map ++ Map('infile -> string), list.tail)
      case option :: tail => println("Unknown option "+option)
                             sys.exit(1)
    }
  }
  val options = nextOption(Map(),arglist)

  if(!options.isDefinedAt('entregadores)) {
    println("Arquivo de entregadores e obrigatorio")
    sys.exit(1)
  }
  val fileEntregadores = options.get('entregadores).mkString
  if(!new File(fileEntregadores).exists) {
    println("Arquivo "+fileEntregadores+" não existe")
    sys.exit(1)
  }

  if(!options.isDefinedAt('cardapio)) {
    println("Arquivo de cardapio e obrigatorio")
    sys.exit(1)
  }
  val fileCardapio = options.get('cardapio).mkString
  if(!new File(fileCardapio).exists) {
    println("Arquivo "+fileCardapio+" não existe")
    sys.exit(1)
  }

  def csvReader(file:String, delimiter:String): List[List[String]] = {
    val list = Source.fromFile(file).getLines().toList
    val header = list(0).split(delimiter).toList
    list.drop(1).map {_.split(delimiter).toList}
  }

  val entregadores = new Circular[Entregador]
  for(entregador <- csvReader(fileEntregadores, "[;]").sortBy(_(1))){
    entregadores.add(new Entregador(entregador(0), entregador(1)))
  }

  val items = new ArrayBuffer[Item]
  for(item <- csvReader(fileCardapio, "[;]").sortBy(_(0).toInt)){
    items.append(new Item(item(0), item(2), item(3).replace(',','.').toFloat))
  }

  val diskRango = new DiskRango(entregadores, items.toList)

  def printlnCardapio(items: List[Item]) = {
    val list = items.map{_.toList}
    val table = Tabulator.format(List(List("Cod", "Descricao", "Preço"))++list)
    println(table)
  }

  println("Telefone (0 para sair):")
  var telefone = readLine
  while(telefone == ""){
    println("Telefone (0 para sair):")
    telefone = readLine
  }
  while(telefone != "0") {
    val cliente = diskRango.buscaCliente(telefone)

    val pedido = new Pedido(cliente)
    println("Adicionar item número (? para cardápio, 0 para finalizar):")
    var resposta = readLine
    while(resposta != "0"){
      resposta match {
        case "?" => printlnCardapio(diskRango.items)
        case cod:String  if(diskRango.hasItem(cod)) => val item = diskRango.loadItem(cod)
          println("\tAdicionando \"" + item.nome + "\" ao pedido. Quantidade:")
          try { 
            val quantidade = readLine.toInt
            pedido.addItem(quantidade, item)
          } catch {
            case e: NumberFormatException => println("Quantidade invalida, item não adicionado")
          }
        case _ => println("\tNão há item com código " + resposta + " no cardápio.")
      }
      println("Adicionar item número (? para cardápio, 0 para finalizar):")
      resposta = readLine
    }
    val total = pedido.total()
    println("Total: R$ " + "%.2f".format(total).replace('.',',') + ". Confirma?")
    resposta = readLine
    if(resposta != "S" | resposta != "s"){
      diskRango.pedidos.append(pedido)
      entregadores.next.entregaPedido(pedido)
    } else println("Pedido descartado!")
    
    println("Telefone (0 para sair):")
    telefone = readLine
    while(telefone == ""){
      println("Telefone (0 para sair): ")
      telefone = readLine
    }
  }

  if(options.isDefinedAt('pedidos)) {
    val filePedidos = options.get('pedidos).mkString
    val writerPedidos = new PrintWriter(new File(filePedidos))
    
    writerPedidos.write("Nome;Telefone;Data;Hora;Valor total\n")
    for(pedido <- diskRango.pedidos) writerPedidos.write(pedido.toString+"\n")
    writerPedidos.close
  }
}